<?php declare(strict_types=1);

namespace App\Http\Middleware;

use App\Enums\CacheTag;
use App\Repositories\ProjectRepository;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Symfony\Component\HttpFoundation\Response;
use Closure;

class Project
{
    public function __construct(
        private readonly ProjectRepository $projectRepository,
    ) { }

    public function handle(Request $request, Closure $next): Response
    {
        $projectCode = $request->route()?->parameter('project');
        if ($projectCode === null) {
            abort(404);
        }

        $seconds = 3600;
        $project = CacheTag::Project->getCache()->remember(self::class . $projectCode, $seconds, function () use ($projectCode) {
            return $this->projectRepository->getProjectByCode($projectCode) ?? false;
        });
        if ($project === false) {
            abort(404);
        }

        unset($request->route()->parameters['project']);

        $routeName = $request->route()->getName();
        if (
            $routeName !== null
            && $project->http_host !== null
            && $project->http_host !== $request->getSchemeAndHttpHost()
        ) {
            $routeName = Str::of($routeName)->replaceStart('project.', '')->toString();
            $route = \route($routeName, $request->route()->parameters(), false);
            return redirect($project->http_host . $route, 302);
        }

        $request->merge(['project' => $project]);

        return $next($request);
    }
}
