<?php declare(strict_types=1);

namespace App\Http\Middleware;

use App\Enums\CacheTag;
use App\Repositories\ProjectRepository;
use Closure;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

final class ProjectDomain
{
    public function __construct(
        private readonly ProjectRepository $projectRepository,
    ) { }

    public function handle(Request $request, Closure $next): Response
    {
        $httpHost = $request->getSchemeAndHttpHost();

        $seconds = 3600;
        $project = CacheTag::Project->getCache()->remember(self::class . $httpHost, $seconds, function () use ($httpHost) {
            return $this->projectRepository->getProjectByHttpHost($httpHost) ?? false;
        });
        if ($project === false) {
            $project = null;
        }
        $request->merge(['project' => $project]);

        return $next($request);
    }
}
